package org.artifactory.engine;

import edu.emory.mathcs.backport.java.util.Collections;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.request.ArtifactoryRequestBase;
import org.artifactory.common.ConstantValues;
import org.artifactory.fs.RepoResource;
import org.artifactory.resource.RepoResourceInfo;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.jetbrains.annotations.NotNull;
import org.mockito.Mockito;
import org.testng.annotations.AfterClass;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;

import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

public class EtagMatchTest extends ArtifactoryHomeBoundTest {

    @AfterClass
    public void unbind() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @DataProvider(name = "data")
    public Object[][] data() {
        return new Object[][] {
                // legacy cases
                {"sha-1", "sha-1", "md5", false, false},
                {"sha-1", "sha-2", "md5", false, true},
                {"md5", "sha-1", "md5", false, true},
                {"md6", "sha-1", "md5", false, true},
                // include md5 enabled
                {"sha-1", "sha-1", "md5", true, false},
                {"sha-1", "sha-2", "md5", true, true},
                {"md5", "sha-1", "md5", true, false},
                {"\"md5\"", "sha-1", "md5", true, false},
                {"W/\"md5\"", "sha-1", "md5", true, false},
                {"W/md5", "sha-1", "md5", true, false},
                {"md5", "sha-1", "md6", true, true},
        };
    }

    @Test(dataProvider = "data")
    public void test(String etag, String sha1, String md5, boolean includeMd5, boolean expected) {
        setStringSystemProperty(ConstantValues.includeMd5IntoETagCheck, Boolean.toString(includeMd5));
        DownloadServiceImpl downloadService = new DownloadServiceImpl();
        RequestStub request = new RequestStub();
        request.etag = etag;
        RepoResource resource = Mockito.mock(RepoResource.class);
        RepoResourceInfo repoResourceInfo = Mockito.mock(RepoResourceInfo.class);
        when(repoResourceInfo.getSha1()).thenReturn(sha1);
        when(repoResourceInfo.getMd5()).thenReturn(md5);
        when(resource.getInfo()).thenReturn(repoResourceInfo);
        assertEquals(downloadService.isNonMatch(request, resource), expected);
    }

    private static class RequestStub extends ArtifactoryRequestBase {

        String etag;

        @Override
        public boolean isRecursive() {
            return false;
        }

        @Override
        public boolean isFromAnotherArtifactory() {
            return false;
        }

        @Override
        public boolean isHeadOnly() {
            return false;
        }

        @Override
        public long getLastModified() {
            return 0;
        }

        @Override
        public long getIfModifiedSince() {
            return 0;
        }

        @Override
        public boolean hasIfModifiedSince() {
            return false;
        }

        @Override
        public String getServletContextUrl() {
            return null;
        }

        @Override
        public String getUri() {
            return null;
        }

        @Override
        public InputStream getInputStream() throws IOException {
            return null;
        }

        @Override
        public long getContentLength() {
            return 0;
        }

        @Override
        public String getClientAddress() {
            return null;
        }

        @Override
        public Enumeration<String> getHeadersKeys() {
            return null;
        }

        @Override
        public Enumeration<String> getHeaderValues(@NotNull String headerName) {
            return Collections.enumeration(Collections.singletonList(etag));
        }
    }
}
