package org.artifactory.queue;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.HaAddon;
import org.artifactory.common.ConstantValues;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.queue.services.PersistentQueueCursorServiceImpl;
import org.artifactory.storage.db.event.model.NodeEventCursor;
import org.artifactory.storage.db.event.service.NodeEventCursorService;
import org.artifactory.storage.event.EventsService;
import org.jfrog.common.concurrency.ConflictsGuard;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.openMocks;
import static org.testng.Assert.assertEquals;

public class PersistentQueueCursorServiceImplTest {

    @Mock
    private NodeEventCursorService nodeEventCursorService;

    @Mock
    private EventsService eventsService;

    @Mock
    private AddonsManager addonsManager;

    @Mock
    private ArtifactoryHome home;

    @Mock
    private ArtifactorySystemProperties props;

    @Mock
    private HaAddon haAddon;

    @Mock
    private ConflictsGuard conflictsGuard;

    private PersistentQueueCursorServiceImpl cursorService;

    @BeforeMethod
    private void setup() {
        openMocks(this);

        ArtifactoryHome.bind(home);
        when(home.getArtifactoryProperties()).thenReturn(props);
        when(props.getLongProperty(eq(ConstantValues.persistentQueueRepoMinEventCacheExpireSec))).thenReturn(0L);
        when(addonsManager.addonByType(eq(HaAddon.class))).thenReturn(haAddon);
        when(haAddon.getConflictsGuard(anyString())).thenReturn(conflictsGuard);

        cursorService = new PersistentQueueCursorServiceImpl(nodeEventCursorService, eventsService, addonsManager);

    }

    @AfterMethod
    private void teardown() {
        ArtifactoryHome.unbind();
    }

    @Test
    public void testGetStatusNotAvailable() {


        when(nodeEventCursorService.cursorForOperator(eq("cursor1"))).thenReturn(null);

        QueueStatus status = cursorService.getStatus("repo1", "cursor1");

        assertEquals(status, QueueStatus.NOT_AVAILABLE);

    }

    @DataProvider
    private Object[][] testGetStatusProvider() {
        return new Object[][] {
                {1L, 3L, QueueStatus.DATA_LOST},
                {10L, 5L, QueueStatus.OK},
                {1L, 2L, QueueStatus.OK},
                {2L, 2L, QueueStatus.OK},
        };
    }

    @Test(dataProvider = "testGetStatusProvider")
    public void testGetStatus(long marker, long firstEventId, QueueStatus expectedStatus) {
        NodeEventCursor cursor = Mockito.mock(NodeEventCursor.class);
        when(nodeEventCursorService.cursorForOperator(eq("cursor1"))).thenReturn(cursor);
        when(cursor.getEventMarker()).thenReturn(marker);
        when(eventsService.getFirstEventLogId(eq("repo1"))).thenReturn(firstEventId);
        QueueStatus status = cursorService.getStatus("repo1", "cursor1");
        assertEquals(status, expectedStatus);
    }

    @Test
    public void testReset() {
        cursorService.resetMarker("repo1", "cursor1");
        verify(eventsService, times(1)).getLastEventLogId();
    }
}
