package org.artifactory.opentracing;

import io.opentracing.Scope;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;

import java.io.Closeable;

import static org.artifactory.opentracing.ArtifactoryTracer.UBER_TRACE_ID;

/**
 * Resembles a server-side Tracing Scope. The scope is the product of a created span.
 * The span is started and finished via an instance of this class
 *
 * @author Uriah Levy
 */
public class ServerTracingScope implements Closeable {
    private static final Logger log = LoggerFactory.getLogger(ServerTracingScope.class);
    private Scope tracingScope;

    ServerTracingScope(Scope tracingScope, String traceId) {
        this.tracingScope = tracingScope;
        MDC.put(UBER_TRACE_ID, traceId);
        log.trace("Started a new server Tracing Span with trace-id '{}'", traceId);
    }

    @Override
    public void close() {
        tracingScope.close();
        MDC.remove(UBER_TRACE_ID);
    }
}
