package org.artifactory.storage.db.conversion.version.v229;

import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.jfrog.storage.util.DbUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.SQLException;
import java.util.function.BiPredicate;


/**
 * @author andreik
 */
public class V229ConversionPredicateNodeEventsTempIndex implements ConversionPredicate {

    private static final Logger log = LoggerFactory.getLogger(V229ConversionPredicateNodeEventsTempIndex.class);

    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            try {
                boolean indexExists = DbUtils.withConnection(jdbcHelper,
                        conn -> DbUtils.indexExists(jdbcHelper, conn, dbType, "node_events_tmp", "tmp_event_id",
                                "node_events_tmp_event_id_idx", true));
                log.debug("Index: {}, exists: {}", "node_events_tmp_event_id_idx", indexExists);
                return !indexExists;
            } catch (SQLException e) {
                log.error("Cannot run conversion 'v229' - Failed to check if index exists: ", e);
            }
            return false;
        };
    }
}
