package org.artifactory.storage.db.conversion.version.v225;

import org.jfrog.storage.DbType;
import org.jfrog.storage.JFrogDataSource;
import org.jfrog.storage.JdbcHelper;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.function.BiPredicate;

/**
 * @author Alexei Vainshtein
 */
public class V225ConversionPredicateTest {

    private JdbcHelper prepareSchema() throws SQLException {
        JdbcHelper jdbcHelper = Mockito.mock(JdbcHelper.class);
        JFrogDataSource dataSource = Mockito.mock(JFrogDataSource.class);
        Connection connection = Mockito.mock(Connection.class);
        Mockito.when(connection.getSchema()).thenReturn("schemaName");
        Mockito.when(dataSource.getConnection()).thenReturn(connection);
        Mockito.when(jdbcHelper.getDataSource()).thenReturn(dataSource);
        return jdbcHelper;
    }

    @Test
    public void testCondition() {
        V225ConversionPredicate v225ConversionPredicate = new V225ConversionPredicate("", "");
        BiPredicate<JdbcHelper, DbType> condition = v225ConversionPredicate.condition();
        Assert.assertFalse(condition.test(null, DbType.DERBY));
        Assert.assertFalse(condition.test(null, DbType.ORACLE));
        Assert.assertFalse(condition.test(null, DbType.MARIADB));
        Assert.assertFalse(condition.test(null, DbType.MSSQL));
        Assert.assertFalse(condition.test(null, DbType.MYSQL));
    }

    @Test
    public void testConditionForPostgresqlNoIndex() throws SQLException {
        V225ConversionPredicate v225ConversionPredicate = new V225ConversionPredicate("", "");
        BiPredicate<JdbcHelper, DbType> condition = v225ConversionPredicate.condition();
        JdbcHelper jdbcHelper = prepareSchema();
        ResultSet rs = Mockito.mock(ResultSet.class);
        Mockito.when(jdbcHelper.executeSelect(Mockito.anyString(), Mockito.anyString(), Mockito.anyString()))
                .thenReturn(rs);
        Assert.assertTrue(condition.test(jdbcHelper, DbType.POSTGRESQL));
    }

    @Test
    public void testConditionForPostgresqlPartlyIndex() throws SQLException {
        V225ConversionPredicate v225ConversionPredicate = new V225ConversionPredicate("", "check");
        BiPredicate<JdbcHelper, DbType> condition = v225ConversionPredicate.condition();
        JdbcHelper jdbcHelper = prepareSchema();
        ResultSet rs = Mockito.mock(ResultSet.class);
        Mockito.when(rs.next()).thenReturn(true);
        Mockito.when(rs.getString("indexdef")).thenReturn("nope");
        Mockito.when(jdbcHelper.executeSelect(Mockito.anyString(), Mockito.anyString(), Mockito.anyString()))
                .thenReturn(rs);
        Assert.assertTrue(condition.test(jdbcHelper, DbType.POSTGRESQL));
    }

    @Test
    public void testConditionForPostgresqlIndexExists() throws SQLException {
        V225ConversionPredicate v225ConversionPredicate = new V225ConversionPredicate("", "check");
        BiPredicate<JdbcHelper, DbType> condition = v225ConversionPredicate.condition();
        JdbcHelper jdbcHelper = prepareSchema();
        ResultSet rs = Mockito.mock(ResultSet.class);
        Mockito.when(rs.next()).thenReturn(true);
        Mockito.when(rs.getString("indexdef")).thenReturn("check");
        Mockito.when(jdbcHelper.executeSelect(Mockito.anyString(), Mockito.anyString(), Mockito.anyString()))
                .thenReturn(rs);
        Assert.assertFalse(condition.test(jdbcHelper, DbType.POSTGRESQL));
    }
}