package org.artifactory.ui.rest.service.admin.configuration.projects;

import org.artifactory.addon.build.BuildAddon;
import org.artifactory.api.build.BuildService;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.projects.ProjectsService;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.ui.rest.model.admin.configuration.projects.UnassignedResourceStats;
import org.jfrog.access.common.ResourceType;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import java.util.List;
import java.util.Map;
import java.util.Set;

import static org.mockito.ArgumentMatchers.anyBoolean;
import static org.mockito.ArgumentMatchers.anySet;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class GetUnassignedResourcesStatsTest {

    private final static String UNASSIGNED_REPO = "repoA";
    private final static String ASSIGNED_REPO = "repoB";
    private final static String ASSIGNED_REPO2 = "virtual1";

    private GetUnassignedResourcesStats getUnassignedResourcesStats;
    @Mock
    private RepositoryService repositoryService;
    @Mock
    private ProjectsService projectsService;
    @Mock
    private BuildService buildService;

    @Before
    public void setUp() {
        MockitoAnnotations.openMocks(this);
        getUnassignedResourcesStats = Mockito.spy(new GetUnassignedResourcesStats());
        getUnassignedResourcesStats.setBuildService(buildService);
        getUnassignedResourcesStats.setProjectsService(projectsService);
        getUnassignedResourcesStats.setRepositoryService(repositoryService);

        when(repositoryService.getAllProjectSupportedRepoKeys())
                .thenReturn(Set.of(UNASSIGNED_REPO, ASSIGNED_REPO, ASSIGNED_REPO2));
        when(projectsService.getRepoKeyToProjectMap(anySet(), anyBoolean())).thenReturn(
                Map.of(UNASSIGNED_REPO, ProjectsService.DEFAULT_PROJECT, ASSIGNED_REPO, "proj1", ASSIGNED_REPO2,
                        "proj2"));
        when(buildService.getTotalBuildsCount()).thenReturn(100);
        when(buildService.getBuildInfoRepoForDefaultProject()).thenReturn(BuildAddon.BUILD_INFO_REPO_NAME);
        when(buildService.getBuildsCountByRepoKey(BuildAddon.BUILD_INFO_REPO_NAME)).thenReturn(30);
        when(buildService.getUniqueBuildsCountByRepoKey(BuildAddon.BUILD_INFO_REPO_NAME)).thenReturn(20);
    }

    @Test
    public void testGetUnassignedResourcesStats() {
        ArtifactoryRestRequest restRequest = mock(ArtifactoryRestRequest.class);
        ArtifactoryRestResponse response = new ArtifactoryRestResponse();

        getUnassignedResourcesStats.execute(restRequest, response);
        List<UnassignedResourceStats> results = (List<UnassignedResourceStats>) response.getIModel();
        Assert.assertEquals(2, results.size());

        UnassignedResourceStats repoStats = results.get(0);
        Assert.assertEquals(ResourceType.REPO.getKey(), repoStats.getResourceType());
        Assert.assertEquals(1, repoStats.getUnassignedCount());
        Assert.assertEquals(3, repoStats.getTotalCount());

        UnassignedResourceStats buildsStats = results.get(1);
        Assert.assertEquals(ResourceType.BUILD.getKey(), buildsStats.getResourceType());
        Assert.assertEquals(30, buildsStats.getUnassignedCount());
        Assert.assertEquals(20, buildsStats.getUnassignedUniqueCount().intValue());
        Assert.assertEquals(100, buildsStats.getTotalCount());
    }
}