package org.artifactory.rest.services.artifact.lifecycle;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.artifact.lifecycle.ArtifactLifecycleAddon;
import org.artifactory.addon.artifact.lifecycle.ArtifactLifecyclePolicyBaseModel;
import org.artifactory.addon.artifact.lifecycle.PolicyTranslator;
import org.artifactory.descriptor.artifact.lifecycle.ArtifactLifecyclePolicyDescriptor;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;

import static org.artifactory.addon.artifact.lifecycle.ArtifactLifecycleOperationType.ARCHIVE;

/**
 * @author Omri Ziv
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class CreateArtifactLifecyclePolicyService implements RestService<ArtifactLifecyclePolicyBaseModel> {

    @Autowired
    private AddonsManager addonsManager;

    @Override
    public void execute(ArtifactoryRestRequest<ArtifactLifecyclePolicyBaseModel> request, RestResponse response) {
        ArtifactLifecyclePolicyBaseModel policyModelBase = request.getImodel();
        String policyKey = policyModelBase.getKey();
        ArtifactLifecyclePolicyDescriptor
                policyDescriptor = PolicyTranslator.toPolicyDescriptor(policyModelBase, ARCHIVE);
        ArtifactLifecyclePolicyBaseModel policyRetrieved = addonsManager.addonByType(ArtifactLifecycleAddon.class)
                .addPolicy(policyKey, policyDescriptor);
        response.info("Successfully added artifact lifecycle policy " + policyRetrieved.getKey());
        response.responseCode(HttpServletResponse.SC_CREATED);
    }
}
